@@ -43,7 +43,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 419592 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 
 #include "asterisk/module.h"
 #include "asterisk/lock.h"
@@ -126,6 +126,9 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision: 419592 $")
 						Voice Duration - Greeting.
 					</value>
 					<value name="MAXWORDLENGTH">
+						Word Length - max length of a single word.
+					</value>
+					<value name="MAXWORDS">
 						Word Count - maximum number of words.
 					</value>	
 				</variable>
