@@ -763,7 +763,25 @@ ALTER TABLE ps_contacts ADD COLUMN qualify_timeout INTEGER;
 
 ALTER TABLE ps_globals ADD COLUMN max_initial_qualify_time INTEGER;
 
-INSERT INTO alembic_version (version_num) VALUES ('a541e0b5e89');
+-- Running upgrade a541e0b5e89 -> 28b8e71e541f
+
+ALTER TABLE ps_endpoints ADD COLUMN g726_non_standard yesno_values;
+
+-- Running upgrade 28b8e71e541f -> 498357a710ae
+
+ALTER TABLE ps_endpoints ADD COLUMN rtp_keepalive INTEGER;
+
+-- Running upgrade 498357a710ae -> 26f10cadc157
+
+ALTER TABLE ps_endpoints ADD COLUMN rtp_timeout INTEGER;
+
+ALTER TABLE ps_endpoints ADD COLUMN rtp_timeout_hold INTEGER;
+
+-- Running upgrade 26f10cadc157 -> 154177371065
+
+ALTER TABLE ps_globals ADD COLUMN default_from_user VARCHAR(80);
+
+INSERT INTO alembic_version (version_num) VALUES ('154177371065');
 
 COMMIT;
 
