@@ -56,30 +56,54 @@ static int persistent_endpoint_cmp(void *obj, void *arg, int flags)
 	return !strcmp(ast_endpoint_get_resource(persistent1->endpoint), id) ? CMP_MATCH | CMP_STOP : 0;
 }
 
+/*! \brief Structure for communicating contact status to
+ * persistent_endpoint_update_state from the contact/contact_status
+ * observers.
+ */
+struct sip_contact_status {
+	char *uri;
+	enum ast_sip_contact_status_type status;
+	int64_t rtt;
+};
+
 /*! \brief Callback function for changing the state of an endpoint */
-static int persistent_endpoint_update_state(void *obj, void *arg, int flags)
+static int persistent_endpoint_update_state(void *obj, void *arg, void *data, int flags)
 {
 	struct sip_persistent_endpoint *persistent = obj;
 	struct ast_endpoint *endpoint = persistent->endpoint;
 	char *aor = arg;
+	struct sip_contact_status *status = data;
 	struct ao2_container *contacts;
 	struct ast_json *blob;
 	struct ao2_iterator i;
 	struct ast_sip_contact *contact;
 	enum ast_endpoint_state state = AST_ENDPOINT_OFFLINE;
 
-	if (!ast_strlen_zero(aor) && !strstr(persistent->aors, aor)) {
-		return 0;
-	}
+	if (!ast_strlen_zero(aor)) {
+		if (!strstr(persistent->aors, aor)) {
+			return 0;
+		}
 
+		if (status) {
+			char rtt[32];
+			snprintf(rtt, 31, "%" PRId64, status->rtt);
+			blob = ast_json_pack("{s: s, s: s, s: s, s: s, s: s}",
+				"contact_status", ast_sip_get_contact_status_label(status->status),
+				"aor", aor,
+				"uri", status->uri,
+				"roundtrip_usec", rtt,
+				"endpoint_name", ast_endpoint_get_resource(endpoint));
+			ast_endpoint_blob_publish(endpoint, ast_endpoint_contact_state_type(), blob);
+			ast_json_unref(blob);
+		}
+	}
 	/* Find all the contacts for this endpoint.  If ANY are available,
 	 * mark the endpoint as ONLINE.
 	 */
 	contacts = ast_sip_location_retrieve_contacts_from_aor_list(persistent->aors);
 	if (contacts) {
 		i = ao2_iterator_init(contacts, 0);
-		while ((contact = ao2_iterator_next(&i))
-			&& state == AST_ENDPOINT_OFFLINE) {
+		while (state == AST_ENDPOINT_OFFLINE && (contact = ao2_iterator_next(&i))) {
 			struct ast_sip_contact_status *contact_status;
 			const char *contact_id = ast_sorcery_object_get_id(contact);
 
@@ -121,11 +145,28 @@ static int persistent_endpoint_update_state(void *obj, void *arg, int flags)
 /*! \brief Function called when stuff relating to a contact happens (created/deleted) */
 static void persistent_endpoint_contact_created_observer(const void *object)
 {
-	char *id = ast_strdupa(ast_sorcery_object_get_id(object)), *aor = NULL;
+	const struct ast_sip_contact *contact = object;
+	char *id = ast_strdupa(ast_sorcery_object_get_id(contact));
+	char *aor = NULL;
+	char *contact_uri = NULL;
+	struct sip_contact_status status;
 
-	aor = strsep(&id, ";@");
+	aor = id;
+	/* Dynamic contacts are delimited with ";@" and static ones with "@@" */
+	if ((contact_uri = strstr(id, ";@")) || (contact_uri = strstr(id, "@@"))) {
+		*contact_uri = '\0';
+		contact_uri += 2;
+	} else {
+		contact_uri = id;
+	}
 
-	ao2_callback(persistent_endpoints, OBJ_NODATA, persistent_endpoint_update_state, aor);
+	status.uri = contact_uri;
+	status.status = CREATED;
+	status.rtt = 0;
+
+	ast_verb(1, "Contact %s/%s has been created\n", aor, contact_uri);
+
+	ao2_callback_data(persistent_endpoints, OBJ_NODATA, persistent_endpoint_update_state, aor, &status);
 }
 
 /*! \brief Function called when stuff relating to a contact happens (created/deleted) */
@@ -133,20 +174,25 @@ static void persistent_endpoint_contact_deleted_observer(const void *object)
 {
 	char *id = ast_strdupa(ast_sorcery_object_get_id(object));
 	char *aor = NULL;
-	char *contact = NULL;
+	char *contact_uri = NULL;
+	struct sip_contact_status status;
 
 	aor = id;
 	/* Dynamic contacts are delimited with ";@" and static ones with "@@" */
-	if ((contact = strstr(id, ";@")) || (contact = strstr(id, "@@"))) {
-		*contact = '\0';
-		contact += 2;
+	if ((contact_uri = strstr(id, ";@")) || (contact_uri = strstr(id, "@@"))) {
+		*contact_uri = '\0';
+		contact_uri += 2;
 	} else {
-		contact = id;
+		contact_uri = id;
 	}
 
-	ast_verb(1, "Contact %s/%s is now Unavailable\n", aor, contact);
+	ast_verb(1, "Contact %s/%s has been deleted\n", aor, contact_uri);
 
-	ao2_callback(persistent_endpoints, OBJ_NODATA, persistent_endpoint_update_state, aor);
+	status.uri = contact_uri;
+	status.status = REMOVED;
+	status.rtt = 0;
+
+	ao2_callback_data(persistent_endpoints, OBJ_NODATA, persistent_endpoint_update_state, aor, &status);
 }
 
 /*! \brief Observer for contacts so state can be updated on respective endpoints */
@@ -161,23 +207,32 @@ static void persistent_endpoint_contact_status_observer(const void *object)
 	const struct ast_sip_contact_status *contact_status = object;
 	char *id = ast_strdupa(ast_sorcery_object_get_id(object));
 	char *aor = NULL;
-	char *contact = NULL;
+	char *contact_uri = NULL;
+	struct sip_contact_status status;
 
-	/* If rtt_start is set (this is the outgoing OPTIONS) or
-	 * there's no status change, ignore.
-	 */
-	if (contact_status->rtt_start.tv_sec > 0
-		|| contact_status->status == contact_status->last_status) {
+	/* If rtt_start is set (this is the outgoing OPTIONS), ignore. */
+	if (contact_status->rtt_start.tv_sec > 0) {
 		return;
 	}
 
 	aor = id;
 	/* Dynamic contacts are delimited with ";@" and static ones with "@@" */
-	if ((contact = strstr(id, ";@")) || (contact = strstr(id, "@@"))) {
-		*contact = '\0';
-		contact += 2;
+	if ((contact_uri = strstr(id, ";@")) || (contact_uri = strstr(id, "@@"))) {
+		*contact_uri = '\0';
+		contact_uri += 2;
 	} else {
-		contact = id;
+		contact_uri = id;
+	}
+
+	if (contact_status->status == contact_status->last_status) {
+		ast_debug(3, "Contact %s status didn't change: %s, RTT: %.3f msec\n",
+			contact_uri, ast_sip_get_contact_status_label(contact_status->status),
+			contact_status->rtt / 1000.0);
+		return;
+	} else {
+		ast_verb(1, "Contact %s/%s is now %s.  RTT: %.3f msec\n", aor, contact_uri,
+			ast_sip_get_contact_status_label(contact_status->status),
+			contact_status->rtt / 1000.0);
 	}
 
 	ast_test_suite_event_notify("AOR_CONTACT_UPDATE",
@@ -186,10 +241,11 @@ static void persistent_endpoint_contact_status_observer(const void *object)
 		ast_sorcery_object_get_id(contact_status),
 		ast_sip_get_contact_status_label(contact_status->status));
 
-	ast_verb(1, "Contact %s/%s is now %s\n", aor, contact,
-		ast_sip_get_contact_status_label(contact_status->status));
+	status.uri = contact_uri;
+	status.status = contact_status->status;
+	status.rtt = contact_status->rtt;
 
-	ao2_callback(persistent_endpoints, OBJ_NODATA, persistent_endpoint_update_state, aor);
+	ao2_callback_data(persistent_endpoints, OBJ_NODATA, persistent_endpoint_update_state, aor, &status);
 }
 
 /*! \brief Observer for contacts so state can be updated on respective endpoints */
@@ -346,6 +402,10 @@ int ast_sip_auth_vector_init(struct ast_sip_auth_vector *auths, const char *valu
 	}
 
 	while ((val = strsep(&auth_names, ","))) {
+		if (ast_strlen_zero(val)) {
+			continue;
+		}
+
 		val = ast_strdup(val);
 		if (!val) {
 			goto failure;
@@ -644,7 +704,7 @@ static int media_encryption_handler(const struct aco_option *opt, struct ast_var
 
 static const char *media_encryption_map[] = {
 	[AST_SIP_MEDIA_TRANSPORT_INVALID] = "invalid",
-	[AST_SIP_MEDIA_ENCRYPT_NONE] = "none",
+	[AST_SIP_MEDIA_ENCRYPT_NONE] = "no",
 	[AST_SIP_MEDIA_ENCRYPT_SDES] = "sdes",
 	[AST_SIP_MEDIA_ENCRYPT_DTLS] = "dtls",
 };
@@ -1014,7 +1074,7 @@ static struct ast_endpoint *persistent_endpoint_find_or_create(const struct ast_
 		if (ast_strlen_zero(persistent->aors)) {
 			ast_endpoint_set_state(persistent->endpoint, AST_ENDPOINT_UNKNOWN);
 		} else {
-			persistent_endpoint_update_state(persistent, NULL, 0);
+			persistent_endpoint_update_state(persistent, NULL, NULL, 0);
 		}
 
 		ao2_link_flags(persistent_endpoints, persistent, OBJ_NOLOCK);
@@ -1824,6 +1884,9 @@ int ast_res_pjsip_initialize_configuration(const struct ast_module_info *ast_mod
 	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "use_avpf", "no", OPT_BOOL_T, 1, FLDSET(struct ast_sip_endpoint, media.rtp.use_avpf));
 	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "force_avp", "no", OPT_BOOL_T, 1, FLDSET(struct ast_sip_endpoint, media.rtp.force_avp));
 	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "media_use_received_transport", "no", OPT_BOOL_T, 1, FLDSET(struct ast_sip_endpoint, media.rtp.use_received_transport));
+	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "rtp_keepalive", "0", OPT_UINT_T, 0, FLDSET(struct ast_sip_endpoint, media.rtp.keepalive));
+	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "rtp_timeout", "0", OPT_UINT_T, 0, FLDSET(struct ast_sip_endpoint, media.rtp.timeout));
+	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "rtp_timeout_hold", "0", OPT_UINT_T, 0, FLDSET(struct ast_sip_endpoint, media.rtp.timeout_hold));
 	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "one_touch_recording", "no", OPT_BOOL_T, 1, FLDSET(struct ast_sip_endpoint, info.recording.enabled));
 	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "inband_progress", "no", OPT_BOOL_T, 1, FLDSET(struct ast_sip_endpoint, inband_progress));
 	ast_sorcery_object_field_register_custom(sip_sorcery, "endpoint", "call_group", "", group_handler, callgroup_to_str, NULL, 0, 0);
@@ -1866,10 +1929,11 @@ int ast_res_pjsip_initialize_configuration(const struct ast_module_info *ast_mod
 	ast_sorcery_object_field_register_custom(sip_sorcery, "endpoint", "dtls_fingerprint", "", dtls_handler, dtlsfingerprint_to_str, NULL, 0, 0);
 	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "srtp_tag_32", "no", OPT_BOOL_T, 1, FLDSET(struct ast_sip_endpoint, media.rtp.srtp_tag_32));
 	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "media_encryption_optimistic", "no", OPT_BOOL_T, 1, FLDSET(struct ast_sip_endpoint, media.rtp.encryption_optimistic));
+	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "g726_non_standard", "no", OPT_BOOL_T, 1, FLDSET(struct ast_sip_endpoint, media.g726_non_standard));
 	ast_sorcery_object_field_register_custom(sip_sorcery, "endpoint", "redirect_method", "user", redirect_handler, NULL, NULL, 0, 0);
 	ast_sorcery_object_field_register_custom(sip_sorcery, "endpoint", "set_var", "", set_var_handler, set_var_to_str, set_var_to_vl, 0, 0);
-	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "message_context", "", OPT_STRINGFIELD_T, 1, STRFLDSET(struct ast_sip_endpoint, message_context));
-	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "accountcode", "", OPT_STRINGFIELD_T, 1, STRFLDSET(struct ast_sip_endpoint, accountcode));
+	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "message_context", "", OPT_STRINGFIELD_T, 0, STRFLDSET(struct ast_sip_endpoint, message_context));
+	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "accountcode", "", OPT_STRINGFIELD_T, 0, STRFLDSET(struct ast_sip_endpoint, accountcode));
 
 	if (ast_sip_initialize_sorcery_transport()) {
 		ast_log(LOG_ERROR, "Failed to register SIP transport support with sorcery\n");
