@@ -354,7 +354,7 @@ static int new_subscribe(struct ast_sip_endpoint *endpoint,
 {
 	if (!ast_exists_extension(NULL, endpoint->context, resource, PRIORITY_HINT, NULL)) {
 		ast_log(LOG_NOTICE, "Extension state subscription failed: Extension %s does not exist in context '%s' or has no associated hint\n",
-			endpoint->context, resource);
+			resource, endpoint->context);
 		return 404;
 	}
 
