@@ -40,7 +40,7 @@
 #include "asterisk/app.h"
 
 struct mwi_subscription;
-AO2_GLOBAL_OBJ_STATIC(unsolicited_mwi);
+static struct ao2_container *unsolicited_mwi;
 
 #define STASIS_BUCKETS 13
 #define MWI_BUCKETS 53
@@ -138,9 +138,17 @@ static struct mwi_stasis_subscription *mwi_stasis_subscription_alloc(const char
 
 	/* Safe strcpy */
 	strcpy(mwi_stasis_sub->mailbox, mailbox);
+
+	ast_debug(3, "Creating stasis MWI subscription to mailbox %s for endpoint %s\n",
+		mailbox, mwi_sub->id);
 	ao2_ref(mwi_sub, +1);
-	ast_debug(3, "Creating stasis MWI subscription to mailbox %s for endpoint %s\n", mailbox, mwi_sub->id);
 	mwi_stasis_sub->stasis_sub = stasis_subscribe_pool(topic, mwi_stasis_cb, mwi_sub);
+	if (!mwi_stasis_sub->stasis_sub) {
+		/* Failed to subscribe. */
+		ao2_ref(mwi_stasis_sub, -1);
+		ao2_ref(mwi_sub, -1);
+		mwi_stasis_sub = NULL;
+	}
 	return mwi_stasis_sub;
 }
 
@@ -297,7 +305,7 @@ static int mwi_sub_cmp(void *obj, void *arg, int flags)
 
 static int get_message_count(void *obj, void *arg, int flags)
 {
-	RAII_VAR(struct stasis_message *, msg, NULL, ao2_cleanup);
+	struct stasis_message *msg;
 	struct mwi_stasis_subscription *mwi_stasis = obj;
 	struct ast_sip_message_accumulator *counter = arg;
 	struct ast_mwi_state *mwi_state;
@@ -310,6 +318,9 @@ static int get_message_count(void *obj, void *arg, int flags)
 	mwi_state = stasis_message_data(msg);
 	counter->old_msgs += mwi_state->old_msgs;
 	counter->new_msgs += mwi_state->new_msgs;
+
+	ao2_ref(msg, -1);
+
 	return 0;
 }
 
@@ -471,7 +482,7 @@ static int unsubscribe_stasis(void *obj, void *arg, int flags)
 	struct mwi_stasis_subscription *mwi_stasis = obj;
 	if (mwi_stasis->stasis_sub) {
 		ast_debug(3, "Removing stasis subscription to mailbox %s\n", mwi_stasis->mailbox);
-		mwi_stasis->stasis_sub = stasis_unsubscribe(mwi_stasis->stasis_sub);
+		mwi_stasis->stasis_sub = stasis_unsubscribe_and_join(mwi_stasis->stasis_sub);
 	}
 	return CMP_MATCH;
 }
@@ -479,31 +490,50 @@ static int unsubscribe_stasis(void *obj, void *arg, int flags)
 static void mwi_subscription_shutdown(struct ast_sip_subscription *sub)
 {
 	struct mwi_subscription *mwi_sub;
-	RAII_VAR(struct ast_datastore *, mwi_datastore,
-			ast_sip_subscription_get_datastore(sub, MWI_DATASTORE), ao2_cleanup);
+	struct ast_datastore *mwi_datastore;
 
+	mwi_datastore = ast_sip_subscription_get_datastore(sub, MWI_DATASTORE);
 	if (!mwi_datastore) {
 		return;
 	}
 
 	mwi_sub = mwi_datastore->data;
 	ao2_callback(mwi_sub->stasis_subs, OBJ_UNLINK | OBJ_NODATA | OBJ_MULTIPLE, unsubscribe_stasis, NULL);
+	ast_sip_subscription_remove_datastore(sub, MWI_DATASTORE);
+
+	ao2_ref(mwi_datastore, -1);
 }
 
-static struct ast_datastore_info mwi_ds_info = { };
+static void mwi_ds_destroy(void *data)
+{
+	struct mwi_subscription *sub = data;
+
+	ao2_ref(sub, -1);
+}
+
+static struct ast_datastore_info mwi_ds_info = {
+	.destroy = mwi_ds_destroy,
+};
 
 static int add_mwi_datastore(struct mwi_subscription *sub)
 {
-	RAII_VAR(struct ast_datastore *, mwi_datastore, NULL, ao2_cleanup);
+	struct ast_datastore *mwi_datastore;
+	int res;
 
 	mwi_datastore = ast_sip_subscription_alloc_datastore(&mwi_ds_info, MWI_DATASTORE);
 	if (!mwi_datastore) {
 		return -1;
 	}
+	ao2_ref(sub, +1);
 	mwi_datastore->data = sub;
 
-	ast_sip_subscription_add_datastore(sub->sip_sub, mwi_datastore);
-	return 0;
+	/*
+	 * NOTE:  Adding the datastore to the subscription creates a ref loop
+	 * that must be manually broken.
+	 */
+	res = ast_sip_subscription_add_datastore(sub->sip_sub, mwi_datastore);
+	ao2_ref(mwi_datastore, -1);
+	return res;
 }
 
 /*!
@@ -517,18 +547,12 @@ static int add_mwi_datastore(struct mwi_subscription *sub)
 static int endpoint_receives_unsolicited_mwi_for_mailbox(struct ast_sip_endpoint *endpoint,
 		const char *mailbox)
 {
-	struct ao2_container *unsolicited = ao2_global_obj_ref(unsolicited_mwi);
 	struct ao2_iterator *mwi_subs;
 	struct mwi_subscription *mwi_sub;
 	const char *endpoint_id = ast_sorcery_object_get_id(endpoint);
 	int ret = 0;
 
-	if (!unsolicited) {
-		return 0;
-	}
-
-	mwi_subs = ao2_find(unsolicited, endpoint_id, OBJ_SEARCH_KEY | OBJ_MULTIPLE);
-	ao2_cleanup(unsolicited);
+	mwi_subs = ao2_find(unsolicited_mwi, endpoint_id, OBJ_SEARCH_KEY | OBJ_MULTIPLE);
 
 	if (!mwi_subs) {
 		return 0;
@@ -597,11 +621,15 @@ static int mwi_on_aor(void *obj, void *arg, int flags)
 
 	mailboxes = ast_strdupa(aor->mailboxes);
 	while ((mailbox = strsep(&mailboxes, ","))) {
-		RAII_VAR(struct mwi_stasis_subscription *, mwi_stasis_sub,
-				mwi_stasis_subscription_alloc(mailbox, sub), ao2_cleanup);
-		if (mwi_stasis_sub) {
-			ao2_link(sub->stasis_subs, mwi_stasis_sub);
+		struct mwi_stasis_subscription *mwi_stasis_sub;
+
+		mwi_stasis_sub = mwi_stasis_subscription_alloc(mailbox, sub);
+		if (!mwi_stasis_sub) {
+			continue;
 		}
+
+		ao2_link(sub->stasis_subs, mwi_stasis_sub);
+		ao2_ref(mwi_stasis_sub, -1);
 	}
 
 	return 0;
@@ -617,8 +645,8 @@ static struct mwi_subscription *mwi_create_subscription(
 	}
 
 	if (add_mwi_datastore(sub)) {
-		ast_log(LOG_WARNING, "Unable to allocate datastore on MWI "
-			"subscription from %s\n", sub->id);
+		ast_log(LOG_WARNING, "Unable to add datastore for MWI subscription to %s\n",
+			sub->id);
 		ao2_ref(sub, -1);
 		return NULL;
 	}
@@ -629,25 +657,26 @@ static struct mwi_subscription *mwi_create_subscription(
 static struct mwi_subscription *mwi_subscribe_single(
 	struct ast_sip_endpoint *endpoint, struct ast_sip_subscription *sip_sub, const char *name)
 {
-	RAII_VAR(struct ast_sip_aor *, aor,
-		 ast_sip_location_retrieve_aor(name), ao2_cleanup);
+	struct ast_sip_aor *aor;
 	struct mwi_subscription *sub;
 
+	aor = ast_sip_location_retrieve_aor(name);
 	if (!aor) {
 		/*! I suppose it's possible for the AOR to disappear on us
 		 * between accepting the subscription and sending the first
 		 * NOTIFY...
 		 */
-		ast_log(LOG_WARNING, "Unable to locate aor %s. MWI "
-			"subscription failed.\n", name);
+		ast_log(LOG_WARNING, "Unable to locate aor %s. MWI subscription failed.\n",
+			name);
 		return NULL;
 	}
 
-	if (!(sub = mwi_create_subscription(endpoint, sip_sub))) {
-		return NULL;
+	sub = mwi_create_subscription(endpoint, sip_sub);
+	if (sub) {
+		mwi_on_aor(aor, sub, 0);
 	}
 
-	mwi_on_aor(aor, sub, 0);
+	ao2_ref(aor, -1);
 	return sub;
 }
 
@@ -657,7 +686,6 @@ static struct mwi_subscription *mwi_subscribe_all(
 	struct mwi_subscription *sub;
 
 	sub = mwi_create_subscription(endpoint, sip_sub);
-
 	if (!sub) {
 		return NULL;
 	}
@@ -679,16 +707,15 @@ static int mwi_new_subscribe(struct ast_sip_endpoint *endpoint,
 	}
 
 	aor = ast_sip_location_retrieve_aor(resource);
-
 	if (!aor) {
-		ast_log(LOG_WARNING, "Unable to locate aor %s. MWI "
-			"subscription failed.\n", resource);
+		ast_log(LOG_WARNING, "Unable to locate aor %s. MWI subscription failed.\n",
+			resource);
 		return 404;
 	}
 
 	if (ast_strlen_zero(aor->mailboxes)) {
-		ast_log(LOG_NOTICE, "AOR %s has no configured mailboxes. "
-			"MWI subscription failed\n", resource);
+		ast_log(LOG_NOTICE, "AOR %s has no configured mailboxes. MWI subscription failed.\n",
+			resource);
 		return 404;
 	}
 
@@ -711,12 +738,19 @@ static int mwi_subscription_established(struct ast_sip_subscription *sip_sub)
 	} else {
 		sub = mwi_subscribe_single(endpoint, sip_sub, resource);
 	}
-
 	if (!sub) {
 		ao2_cleanup(endpoint);
 		return -1;
 	}
 
+	if (!ao2_container_count(sub->stasis_subs)) {
+		/*
+		 * We setup no MWI subscriptions so remove the MWI datastore
+		 * to break the ref loop.
+		 */
+		ast_sip_subscription_remove_datastore(sip_sub, MWI_DATASTORE);
+	}
+
 	ao2_cleanup(sub);
 	ao2_cleanup(endpoint);
 	return 0;
@@ -748,16 +782,16 @@ static void *mwi_get_notify_data(struct ast_sip_subscription *sub)
 static void mwi_subscription_mailboxes_str(struct ao2_container *stasis_subs,
 					   struct ast_str **str)
 {
-	int num = ao2_container_count(stasis_subs);
-
+	int is_first = 1;
 	struct mwi_stasis_subscription *node;
 	struct ao2_iterator i = ao2_iterator_init(stasis_subs, 0);
 
 	while ((node = ao2_iterator_next(&i))) {
-		if (--num) {
-			ast_str_append(str, 0, "%s,", node->mailbox);
-		} else {
+		if (is_first) {
+			is_first = 0;
 			ast_str_append(str, 0, "%s", node->mailbox);
+		} else {
+			ast_str_append(str, 0, ",%s", node->mailbox);
 		}
 		ao2_ref(node, -1);
 	}
@@ -768,9 +802,9 @@ static void mwi_to_ami(struct ast_sip_subscription *sub,
 		       struct ast_str **buf)
 {
 	struct mwi_subscription *mwi_sub;
-	RAII_VAR(struct ast_datastore *, mwi_datastore,
-			ast_sip_subscription_get_datastore(sub, MWI_DATASTORE), ao2_cleanup);
+	struct ast_datastore *mwi_datastore;
 
+	mwi_datastore = ast_sip_subscription_get_datastore(sub, MWI_DATASTORE);
 	if (!mwi_datastore) {
 		return;
 	}
@@ -781,6 +815,8 @@ static void mwi_to_ami(struct ast_sip_subscription *sub,
 	ast_str_append(buf, 0, "Mailboxes: ");
 	mwi_subscription_mailboxes_str(mwi_sub->stasis_subs, buf);
 	ast_str_append(buf, 0, "\r\n");
+
+	ao2_ref(mwi_datastore, -1);
 }
 
 static int serialized_notify(void *userdata)
@@ -810,7 +846,9 @@ static int serialized_cleanup(void *userdata)
 static int send_notify(void *obj, void *arg, int flags)
 {
 	struct mwi_subscription *mwi_sub = obj;
-	struct ast_taskprocessor *serializer = mwi_sub->is_solicited ? ast_sip_subscription_get_serializer(mwi_sub->sip_sub) : NULL;
+	struct ast_taskprocessor *serializer = mwi_sub->is_solicited
+		? ast_sip_subscription_get_serializer(mwi_sub->sip_sub)
+		: NULL;
 
 	if (ast_sip_push_task(serializer, serialized_notify, ao2_bump(mwi_sub))) {
 		ao2_ref(mwi_sub, -1);
@@ -836,18 +874,43 @@ static void mwi_stasis_cb(void *userdata, struct stasis_subscription *sub,
 	}
 }
 
+/*! \note Called with the unsolicited_mwi conainer lock held. */
 static int create_mwi_subscriptions_for_endpoint(void *obj, void *arg, int flags)
 {
 	RAII_VAR(struct mwi_subscription *, aggregate_sub, NULL, ao2_cleanup);
 	struct ast_sip_endpoint *endpoint = obj;
-	struct ao2_container *mwi_subscriptions = arg;
-	char *mailboxes;
-	char *mailbox;
+	char *endpoint_aors, *aor_name, *mailboxes, *mailbox;
+	struct ao2_container *contacts = NULL;
 
 	if (ast_strlen_zero(endpoint->subscription.mwi.mailboxes)) {
 		return 0;
 	}
 
+	endpoint_aors = ast_strdupa(endpoint->aors);
+
+	while ((aor_name = strsep(&endpoint_aors, ","))) {
+		RAII_VAR(struct ast_sip_aor *, aor, ast_sip_location_retrieve_aor(aor_name), ao2_cleanup);
+
+		if (!aor) {
+			continue;
+		}
+
+		contacts = ast_sip_location_retrieve_aor_contacts(aor);
+		if (!contacts || (ao2_container_count(contacts) == 0)) {
+			ao2_cleanup(contacts);
+			contacts = NULL;
+			continue;
+		}
+
+		break;
+	}
+
+	if (!contacts) {
+		return 0;
+	}
+
+	ao2_ref(contacts, -1);
+
 	if (endpoint->subscription.mwi.aggregate) {
 		aggregate_sub = mwi_subscription_alloc(endpoint, 0, NULL);
 		if (!aggregate_sub) {
@@ -859,18 +922,20 @@ static int create_mwi_subscriptions_for_endpoint(void *obj, void *arg, int flags
 	while ((mailbox = strsep(&mailboxes, ","))) {
 		struct mwi_subscription *sub = aggregate_sub ?:
 			mwi_subscription_alloc(endpoint, 0, NULL);
-		RAII_VAR(struct mwi_stasis_subscription *, mwi_stasis_sub,
-				mwi_stasis_subscription_alloc(mailbox, sub), ao2_cleanup);
+		struct mwi_stasis_subscription *mwi_stasis_sub;
+
+		mwi_stasis_sub = mwi_stasis_subscription_alloc(mailbox, sub);
 		if (mwi_stasis_sub) {
 			ao2_link(sub->stasis_subs, mwi_stasis_sub);
+			ao2_ref(mwi_stasis_sub, -1);
 		}
-		if (!aggregate_sub) {
-			ao2_link(mwi_subscriptions, sub);
-			ao2_cleanup(sub);
+		if (!aggregate_sub && sub) {
+			ao2_link_flags(unsolicited_mwi, sub, OBJ_NOLOCK);
+			ao2_ref(sub, -1);
 		}
 	}
 	if (aggregate_sub) {
-		ao2_link(mwi_subscriptions, aggregate_sub);
+		ao2_link_flags(unsolicited_mwi, aggregate_sub, OBJ_NOLOCK);
 	}
 	return 0;
 }
@@ -886,13 +951,11 @@ static int unsubscribe(void *obj, void *arg, int flags)
 
 static void create_mwi_subscriptions(void)
 {
-	struct ao2_container *mwi_subscriptions = ao2_container_alloc(MWI_BUCKETS, mwi_sub_hash, mwi_sub_cmp);
-	RAII_VAR(struct ao2_container *, old_mwi_subscriptions, ao2_global_obj_ref(unsolicited_mwi), ao2_cleanup);
-	RAII_VAR(struct ao2_container *, endpoints, ast_sorcery_retrieve_by_fields(
-				ast_sip_get_sorcery(), "endpoint", AST_RETRIEVE_FLAG_MULTIPLE | AST_RETRIEVE_FLAG_ALL, NULL),
-			ao2_cleanup);
+	struct ao2_container *endpoints;
 
-	if (!mwi_subscriptions) {
+	endpoints = ast_sorcery_retrieve_by_fields(ast_sip_get_sorcery(), "endpoint",
+		AST_RETRIEVE_FLAG_MULTIPLE | AST_RETRIEVE_FLAG_ALL, NULL);
+	if (!endpoints) {
 		return;
 	}
 
@@ -902,12 +965,12 @@ static void create_mwi_subscriptions(void)
 	 * and resubscribing, up-to-date mailbox state will be sent out to the endpoint when the
 	 * new stasis subscription is established
 	 */
-	if (old_mwi_subscriptions) {
-		ao2_callback(old_mwi_subscriptions, OBJ_UNLINK | OBJ_NODATA | OBJ_MULTIPLE, unsubscribe, NULL);
-	}
-	ao2_callback(endpoints, OBJ_NODATA, create_mwi_subscriptions_for_endpoint, mwi_subscriptions);
-	ao2_global_obj_replace_unref(unsolicited_mwi, mwi_subscriptions);
-	ao2_ref(mwi_subscriptions, -1);
+	ao2_lock(unsolicited_mwi);
+	ao2_callback(unsolicited_mwi, OBJ_NOLOCK | OBJ_UNLINK | OBJ_NODATA | OBJ_MULTIPLE, unsubscribe, NULL);
+	ao2_callback(endpoints, OBJ_NODATA, create_mwi_subscriptions_for_endpoint, NULL);
+	ao2_unlock(unsolicited_mwi);
+
+	ao2_ref(endpoints, -1);
 }
 
 /*! \brief Function called to send MWI NOTIFY on any unsolicited mailboxes relating to this AOR */
@@ -927,40 +990,55 @@ static int send_contact_notify(void *obj, void *arg, int flags)
 	return 0;
 }
 
-
-/*! \brief Function called when a contact is created or updated */
-static void mwi_contact_changed_observer(const void *object)
+/*! \brief Function called when a contact is updated */
+static void mwi_contact_updated(const void *object)
 {
 	char *id = ast_strdupa(ast_sorcery_object_get_id(object)), *aor = NULL;
-	struct ao2_container *mwi_subscriptions = ao2_global_obj_ref(unsolicited_mwi);
-
-	if (!mwi_subscriptions) {
-		return;
-	}
 
 	aor = strsep(&id, ";@");
 
-	ao2_callback(mwi_subscriptions, OBJ_NODATA, send_contact_notify, aor);
-	ao2_ref(mwi_subscriptions, -1);
+	ao2_callback(unsolicited_mwi, OBJ_NODATA, send_contact_notify, aor);
+}
+
+/*! \brief Function called when a contact is added */
+static void mwi_contact_added(const void *object)
+{
+	const struct ast_sip_contact *contact = object;
+	struct ao2_iterator *mwi_subs;
+	struct mwi_subscription *mwi_sub;
+	const char *endpoint_id = ast_sorcery_object_get_id(contact->endpoint);
+
+	if (ast_strlen_zero(contact->endpoint->subscription.mwi.mailboxes)) {
+		return;
+	}
+
+	ao2_lock(unsolicited_mwi);
+
+	mwi_subs = ao2_find(unsolicited_mwi, endpoint_id, OBJ_SEARCH_KEY | OBJ_MULTIPLE | OBJ_NOLOCK | OBJ_UNLINK);
+	if (mwi_subs) {
+		for (; (mwi_sub = ao2_iterator_next(mwi_subs)); ao2_cleanup(mwi_sub)) {
+			unsubscribe(mwi_sub, NULL, 0);
+		}
+		ao2_iterator_destroy(mwi_subs);
+	}
+
+	create_mwi_subscriptions_for_endpoint(contact->endpoint, NULL, 0);
+
+	ao2_unlock(unsolicited_mwi);
+
+	mwi_contact_updated(object);
 }
 
 /*! \brief Observer for contacts so unsolicited MWI is sent when a contact changes */
 static const struct ast_sorcery_observer mwi_contact_observer = {
-	.created = mwi_contact_changed_observer,
-	.updated = mwi_contact_changed_observer,
+	.created = mwi_contact_added,
+	.updated = mwi_contact_updated,
 };
 
 /*! \brief Task invoked to send initial MWI NOTIFY for unsolicited */
 static int send_initial_notify_all(void *obj)
 {
-	struct ao2_container *mwi_subscriptions = ao2_global_obj_ref(unsolicited_mwi);
-
-	if (!mwi_subscriptions) {
-		return 0;
-	}
-
-	ao2_callback(mwi_subscriptions, OBJ_NODATA, send_notify, NULL);
-	ao2_ref(mwi_subscriptions, -1);
+	ao2_callback(unsolicited_mwi, OBJ_NODATA, send_notify, NULL);
 
 	return 0;
 }
@@ -1000,6 +1078,13 @@ static int load_module(void)
 	if (ast_sip_register_subscription_handler(&mwi_handler)) {
 		return AST_MODULE_LOAD_DECLINE;
 	}
+
+	unsolicited_mwi = ao2_container_alloc(MWI_BUCKETS, mwi_sub_hash, mwi_sub_cmp);
+	if (!unsolicited_mwi) {
+		ast_sip_unregister_subscription_handler(&mwi_handler);
+		return AST_MODULE_LOAD_DECLINE;
+	}
+
 	create_mwi_subscriptions();
 	ast_sorcery_observer_add(ast_sip_get_sorcery(), "contact", &mwi_contact_observer);
 
@@ -1014,12 +1099,9 @@ static int load_module(void)
 
 static int unload_module(void)
 {
-	RAII_VAR(struct ao2_container *, mwi_subscriptions, ao2_global_obj_ref(unsolicited_mwi), ao2_cleanup);
-	if (mwi_subscriptions) {
-		ao2_callback(mwi_subscriptions, OBJ_UNLINK | OBJ_NODATA | OBJ_MULTIPLE, unsubscribe, NULL);
-		ao2_global_obj_release(unsolicited_mwi);
-		ast_sorcery_observer_remove(ast_sip_get_sorcery(), "contact", &mwi_contact_observer);
-	}
+	ao2_callback(unsolicited_mwi, OBJ_UNLINK | OBJ_NODATA | OBJ_MULTIPLE, unsubscribe, NULL);
+	ao2_ref(unsolicited_mwi, -1);
+	ast_sorcery_observer_remove(ast_sip_get_sorcery(), "contact", &mwi_contact_observer);
 	ast_sip_unregister_subscription_handler(&mwi_handler);
 	return 0;
 }
